/* Beispiel fuer Java-Datenstrukturen
 * "Bestellung0" zur Vorlesung "Softwaretechnologie".
 * (c) Heinrich Hussmann, Uwe Aßmann, TU Dresden, 2013 

 * Dies ist eine sehr einfache Version des Beispiels "Bestellung"
 * Es werden nur einfache Arrays verwendet.
 * Dadurch sind einige Operationen nicht oder nur schwer realisierbar.
 * Methode grosseBestellung zeigt die Grenzen des Einsatzs von statisch festen Arrays auf.
 *
 * Untersuche die Unterschiede zwischen normaleBestellung() und
 * großeBestellung()! Warum stürzt letztere Prozedur ab?
 * @Version 2.2 Generics
 */

class Artikel {

  private String name;
  private int preis;
  
  public Artikel (String name, int preis) {
    this.name = name;
    this.preis = preis;
  }
  
  public int preis() {
    return preis;
  }
  
  public String toString() {
    return name;
  }
}


class Bestellposition {

  private Artikel artikel;
  private int anzahl;
  private int preis; // Standard: Artikelpreis, aber Sonderpreise moeglich
    
  public Bestellposition (Artikel artikel, int anzahl) {
    this.artikel = artikel;
    this.anzahl = anzahl;
    this.preis = artikel.preis();
  }
  
  public int einzelpreis() {
    return preis;
  }
  
  public void einzelpreis (int sonderpreis) {
    preis = sonderpreis;
  }
  
  public int positionspreis() {
    return preis*anzahl;
  }
  
  public String toString() {
    return anzahl+" x "+artikel+" Einzelpreis: "+preis+" Summe: "+positionspreis();
  }

}


class Bestellung0 {

    private String kunde;
    private Bestellposition[] liste; // beachte: Array ist nicht flexibel!
    private int anzahl = 0;

    public Bestellung0(String kunde) {
	this.kunde = kunde;
	liste = new Bestellposition[20];
    }
  
    public void neuePosition (Bestellposition b) {
	// was passiert bei mehr als 20 Positionen ?
	liste[anzahl] = b;
	anzahl++;
    }
  
    public void loeschePosition (int pos) {
	// geht mit Arrays nicht einfach zu realisieren !
    }
  
    public void sonderpreis (int pos, int preis) {
	liste[pos].einzelpreis(preis);
    }
  
    public int auftragssumme() {
	int s = 0;
	for(int i=0; i<anzahl; i++)
	    s += liste[i].positionspreis();
	return s;
    }
  
    public void print () {
	System.out.println("Bestellung fuer Kunde "+kunde);
	for(int i=0; i<anzahl; i++)
	    System.out.println(i+". "+liste[i]);
	System.out.println("Auftragssumme: "+auftragssumme());
	System.out.println();
    }
  
  
    /* Main program. */
    public static void main (String[] args) {
	//normaleBestellung();
	grosseBestellung();
    }	
    /* Diese Bestellung hier geht durch. */
    public static void normaleBestellung () {
	Artikel tisch = new Artikel("Tisch",200);
	Artikel stuhl = new Artikel("Stuhl",100);
	Artikel schrank = new Artikel("Schrank",300);
	
	Bestellung0 b1 = new Bestellung0("TUD");
	b1.neuePosition(new Bestellposition(tisch,1));
	b1.neuePosition(new Bestellposition(stuhl,4));
	b1.neuePosition(new Bestellposition(schrank,2));
	b1.print();
	
	System.out.println("Sonderpreis für TUD");
	b1.sonderpreis(1,50);
	b1.print();
    }
  
    /**
     * Analysieren Sie, warum diese Methode das Programm zum Absturz bringt.
     * Was könnte man verbessern?
     */
    public static void grosseBestellung () {
	int i = 0;

	Artikel tisch = new Artikel("Tisch",200);
	Artikel stuhl = new Artikel("Stuhl",100);
	Artikel schrank = new Artikel("Schrank",300);
	
	Bestellung0 b1 = new Bestellung0("TUD");
	b1.neuePosition(new Bestellposition(tisch,1));
	b1.neuePosition(new Bestellposition(schrank,2));
	
	// 50 einzelne Positionen Stühle
	while (i < 50) {
	    b1.neuePosition(new Bestellposition(stuhl,1));
	    i++;
	}
	b1.print();
	
	System.out.println("Sonderpreis für TUD");
	b1.sonderpreis(1,50);
	b1.print();
    }
}
