/** Beispiel fuer die sofortige Realisierung von UML-Assoziationen, -Aggregationen
 *  und -Kompositionen.
 * "Bestellung1" zur Vorlesung "Softwaretechnologie".
 * (c) Heinrich Hussmann, Uwe Aßmann, TU Dresden, 2013 

 * Diese Version des Beispiels "Bestellung1" arbeitet mit
 * der Java-Collections-Standardklasse ArrayList. 
 * Warum stürzen große Bestellungen nicht ab?
 * 
 * Es wird noch keine Polymorphie für den Container verwendet.
 * @Version 2.1 Generics
 */

import java.util.ArrayList;

class Artikel {

  private String name;
  private int preis;
  
  public Artikel (String name, int preis) {
    this.name = name;
    this.preis = preis;
  }
  
  public int preis() {
    return preis;
  }
  
  public String toString() {
    return name;
  }
}


class Bestellposition {

  private Artikel artikel;
  private int anzahl;
  private int preis; // Standard: Artikelpreis, aber Sonderpreise moeglich
    
  public Bestellposition (Artikel artikel, int anzahl) {
    this.artikel = artikel;
    this.anzahl = anzahl;
    this.preis = artikel.preis();
  }
  
  public int einzelpreis() {
    return preis;
  }
  
  public void einzelpreis (int sonderpreis) {
    preis = sonderpreis;
  }
  
  public int positionspreis() {
    return preis*anzahl;
  }
  
  public String toString() {
    return anzahl+" x "+artikel+" Einzelpreis: "+preis+" Summe: "+positionspreis();
  }

}

class Bestellung1ArrayList {

  private String kunde;
    // Die Deklaration von liste ist spezialisiert udn verwendet noch keine
    // Polymorphie.
  private ArrayList<Bestellposition> liste;

  public Bestellung1ArrayList(String kunde) {
    this.kunde = kunde;
    liste = new ArrayList<Bestellposition>();
  }
  
  public void neuePosition (Bestellposition b) {
    liste.add(b);
  }
  
  public void loeschePosition (int pos) {
    liste.remove(pos);
  }
  
  public void sonderpreis (int pos, int preis) {
      // Ohne Generics müsste hier ein Cast geschrieben werden:
//     ((Bestellposition)liste.get(pos)).einzelpreis(preis);
      // Mit Generics geht hier alles einfacher
      liste.get(pos).einzelpreis(preis);
  }
  
  public int auftragssumme() {
    int s = 0;
    for(int i=0; i<liste.size(); i++)
	// Safe Application Development: 
	// Ohne Generics müsste hier ein Cast geschrieben werden:
//       s += ((Bestellposition)liste.get(i)).positionspreis();
	// Mit Generics geht hier alles einfacher
	s += liste.get(i).positionspreis();
    return s;
  }
  
  public void print () {
    System.out.println("Bestellung fuer Kunde "+kunde);
    for(int i=0; i<liste.size(); i++)
      System.out.println(i+". "+liste.get(i));
    System.out.println("Auftragssumme: "+auftragssumme());
    System.out.println();
  }
  
  
    /* Main program. */
    public static void main (String[] args) {
//  	normaleBestellung();
	// ArrayList kann beliebig große Bestellungen
	grosseBestellung();
    }	

    /* Diese Bestellung ist klein genug und kann auf jeden Fall behandelt werden. */
    public static void normaleBestellung () {
	Artikel tisch = new Artikel("Tisch",200);
	Artikel stuhl = new Artikel("Stuhl",100);
	Artikel schrank = new Artikel("Schrank",300);
	
	Bestellung1ArrayList b1 = new Bestellung1ArrayList("TUD");
	b1.neuePosition(new Bestellposition(tisch,1));
	b1.neuePosition(new Bestellposition(stuhl,4));
	b1.neuePosition(new Bestellposition(schrank,2));
	b1.print();
	
	System.out.println("Sonderpreis für TUD");
	b1.sonderpreis(1,50);
	b1.print();
    }
   
    /**
     * Analysieren Sie, warum diese Methode das Programm NICHT zum Absturz bringt.
     * Wieviel können Sie nun bestellen? Testen Sie aus, wie groß Ihr Heap ist. 
     * Wie können Sie ihn vergrößern?
     */
    public static void grosseBestellung () {
	int i = 0;

	Artikel tisch = new Artikel("Tisch",200);
	Artikel stuhl = new Artikel("Stuhl",100);
	Artikel schrank = new Artikel("Schrank",300);
	
	Bestellung1ArrayList b1 = new Bestellung1ArrayList("TUD");
	b1.neuePosition(new Bestellposition(tisch,1));
	b1.neuePosition(new Bestellposition(schrank,2));
	
	// 50 einzelne Positionen Stühle
	while (i < 5000) {
	    b1.neuePosition(new Bestellposition(stuhl,1));
	    i++;
	}
	b1.print();
	
	System.out.println("Sonderpreis für TUD");
	b1.sonderpreis(1,0);
	b1.sonderpreis(2,0);
	b1.sonderpreis(50,0); // Fünfzigster Stuhl umsonst
	// Versuche, überzuadressieren
//  	b1.sonderpreis(53,0); // Fünfzigster Stuhl umsonst
	b1.print();
    }
}
