/* Beispiel fuer Java-Datenstrukturen
 * "Bestellung" zur Vorlesung "Softwaretechnologie".
 * (c) Heinrich Hussmann, Uwe Aßmann, TU Dresden, 2013 
 
 * Diese Version des Beispiels "BesteLlung" arbeitet mit der
 * Java2-Standardklasse LinkedList.  Fuer den Durchlauf der Liste werden
 * Iteratoren benutzt (Siehe Kapitel Iterators).
 * @Version 2.0 Generics
 */


import java.util.List;
import java.util.LinkedList;
import java.util.Iterator;

class Artikel {

    private String name;
    private int preis;
  
    public Artikel (String name, int preis) {
	this.name = name;
	this.preis = preis;
    }
  
    public int preis() {
	return preis;
    }
  
    public String toString() {
	return name;
    }
}

class Bestellposition {

    private Artikel artikel;
    private int anzahl;
    private int preis; // Standard: Artikelpreis, aber Sonderpreise moeglich
    
    public Bestellposition (Artikel artikel, int anzahl) {
	this.artikel = artikel;
	this.anzahl = anzahl;
	this.preis = artikel.preis();
    }
  
    public int einzelpreis() {
	return preis;
    }
  
    public void einzelpreis (int sonderpreis) {
	preis = sonderpreis;
    }
  
    public int positionspreis() {
	return preis*anzahl;
    }
  
    public String toString() {
	return anzahl+" x "+artikel+" Einzelpreis: "+preis+" Summe: "+positionspreis();
    }

}

class Bestellung1LinkedList {

    private String kunde;
    private List<Bestellposition> liste;

    public Bestellung1LinkedList(String kunde) {
	this.kunde = kunde;
	// Hier ist jetzt polymorph variiert worden: Statt ArrayList wird
	// LinkedList verwendet 
	liste = new LinkedList<Bestellposition>();
    }
  
    public void neuePosition (Bestellposition b) {
	liste.add(b);
    }
  
    public void loeschePosition (int pos) {
	liste.remove(pos);
    }
  
    public void sonderpreis (int pos, int preis) {
	liste.get(pos).einzelpreis(preis);
    }
  
    public int auftragssumme() {
	Iterator<Bestellposition> i = liste.iterator();
	int s = 0;
	while (i.hasNext()) {
// 	    System.out.print("·");
	    s += i.next().positionspreis();
	}
// 	System.out.println(".");
	return s;
    }
  
    public void print () {
	System.out.println("Bestellung fuer Kunde "+kunde);
	Iterator<Bestellposition> i = liste.iterator();
	int pos = 0;
	while (i.hasNext()) {
	    System.out.println(pos+". "+i.next());
	    pos++;
	}
	System.out.println(".");
	System.out.println("Auftragssumme: "+auftragssumme());
	System.out.println();
    }
  
  
    /* Main program. */
    public static void main (String[] args) {
	// 	normaleBestellung();
	// jetzt geht sie durch, die große Bestellung...
	grosseBestellung();
    }	
    /* Diese Bestellung hier geht durch. */
    public static void normaleBestellung () {
	Artikel tisch = new Artikel("Tisch",200);
	Artikel stuhl = new Artikel("Stuhl",100);
	Artikel schrank = new Artikel("Schrank",300);
	
	Bestellung1LinkedList b1 = new Bestellung1LinkedList("TUD");
	b1.neuePosition(new Bestellposition(tisch,1));
	b1.neuePosition(new Bestellposition(stuhl,4));
	b1.neuePosition(new Bestellposition(schrank,2));
	b1.print();
	
	System.out.println("Sonderpreis für TUD");
	b1.sonderpreis(1,50);
	b1.print();
    }
  
    /**
     * Analysieren Sie, warum diese Methode das Programm zum Absturz bringt.
     * Was könnte man verbessern?
     */
    public static void grosseBestellung () {
	int i = 0;

	Artikel tisch = new Artikel("Tisch",200);
	Artikel stuhl = new Artikel("Stuhl",100);
	Artikel schrank = new Artikel("Schrank",300);
	
	Bestellung1LinkedList b1 = new Bestellung1LinkedList("TUD");
	b1.neuePosition(new Bestellposition(tisch,1));
	b1.neuePosition(new Bestellposition(schrank,2));
	
	// 50 einzelne Positionen Stühle
	while (i < 50) {
	    b1.neuePosition(new Bestellposition(stuhl,1));
	    i++;
	}
	b1.print();
	
	System.out.println("Sonderpreis für TUD");
	b1.sonderpreis(1,50);
	b1.print();
    }

}
