/** 
 * 
 * "Bestellung1SpeedTest" zur Vorlesung "Softwaretechnologie".
 * (c) Heinrich Hussmann, Uwe Aßmann, TU Dresden, 2015 

 * Diese Version des Beispiels "Bestellung1SpeedTest" vergleicht die 
 * Implementierungsklassen der Java-Collections-Standardklassen ArrayList,
 * LinkedList und Vector. 
 * Vector is similar to ArrayList, but can be used with multiple parallel threads, because
 * its modifier methods are being synchronized.
 * 
 * @Version 1.0 
 */

import java.util.*;

class Artikel {

  private String name;
  private int preis;
  
  public Artikel (String name, int preis) {
    this.name = name;
    this.preis = preis;
  }
  
  public int preis() {
    return preis;
  }
  
  public String toString() {
    return name;
  }
}


class Bestellposition {

  private Artikel artikel;
  private int anzahl;
  private int preis; // Standard: Artikelpreis, aber Sonderpreise moeglich
    
  public Bestellposition (Artikel artikel, int anzahl) {
    this.artikel = artikel;
    this.anzahl = anzahl;
    this.preis = artikel.preis();
  }
  
  public int einzelpreis() {
    return preis;
  }
  
  public void einzelpreis (int sonderpreis) {
    preis = sonderpreis;
  }
  
  public int positionspreis() {
    return preis*anzahl;
  }
  
  public String toString() {
    return anzahl+" x "+artikel+" Einzelpreis: "+preis+" Summe: "+positionspreis();
  }

}

class Bestellung1SpeedTest {

  private String kunde;
    // Hier verwenden wir eine Polymorphie, d.h. deklarieren eine
    // Schnittstellenoberklasse.
  private List<Bestellposition> liste;

    /** Dieser Konstruktor kann von außen instruiert werden, was er konkret
     * anlegen soll. 
     */ 
    public Bestellung1SpeedTest(String kunde, String whichImplementation) {
	this.kunde = kunde;
	if (whichImplementation.equals("ArrayList")) {
	    liste = new ArrayList<Bestellposition>();
	} else if (whichImplementation.equals("LinkedList")) {
	    liste = new LinkedList<Bestellposition>();
	} else if (whichImplementation.equals("Vector")) {
	    liste = new Vector<Bestellposition>();
	} else {
	    System.err.println("Wrong collection class");
	} 
    }
  
  public void neuePosition (Bestellposition b) {
    liste.add(b);
  }
  
  public void loeschePosition (int pos) {
    liste.remove(pos);
  }
  
  public Bestellposition getPosition (int pos) {
    return liste.get(pos);
  }
  
  public int size () {
    return liste.size();
  }
  
  public void sonderpreis (int pos, int preis) {
      // Ohne Generics müsste hier ein Cast geschrieben werden:
//     ((Bestellposition)liste.get(pos)).einzelpreis(preis);
      // Mit Generics geht hier alles einfacher
      liste.get(pos).einzelpreis(preis);
  }
  
  public int auftragssumme() {
    int s = 0;
    for(int i=0; i<liste.size(); i++)
      // Ohne Generics müsste hier ein Cast geschrieben werden:
//       s += ((Bestellposition)liste.get(i)).positionspreis();
      // Mit Generics geht hier alles einfacher
      s += liste.get(i).positionspreis();
    return s;
  }
  
  public void print () {
    System.out.println("Bestellung fuer Kunde "+kunde);
    for(int i=0; i<liste.size(); i++)
      System.out.println(i+". "+liste.get(i));
    System.out.println("Auftragssumme: "+auftragssumme());
    System.out.println();
  }
  
    /* Main program mit Performance-Test von Listenimplementierungen. */
    public static void main (String[] args) {
	// Insert-orientierter Test
	long Time01 = grosseBestellung("ArrayList");
	long Time02 = grosseBestellung("LinkedList");
	long Time03 = grosseBestellung("Vector");

	// Insert- und Remove-orientierter Test
	long Time11 = fluktuierendeBestellung("ArrayList");
	long Time12 = fluktuierendeBestellung("LinkedList");
	long Time13 = fluktuierendeBestellung("Vector");

	// Insert- und Remove-orientierter Test (von Hinten löschen)
	long Time21 = fluktuierendeBestellungMitLoeschenVonHinten("ArrayList");
	long Time22 = fluktuierendeBestellungMitLoeschenVonHinten("LinkedList");
	long Time23 = fluktuierendeBestellungMitLoeschenVonHinten("Vector");
	
	// Insert- und Lese-orientierter Test
	long Time31 = grosseBestellungMitLesen("ArrayList");
	long Time32 = grosseBestellungMitLesen("LinkedList");
	long Time33 = grosseBestellungMitLesen("Vector");

	System.out.println("INSERT ArrayList: \t"+Time01);
	System.out.println("INSERT LinkedList: \t"+Time02);
	System.out.println("INSERT Vector:     \t"+Time03);
	System.out.println("FLUKTUIEREND ArrayList: \t"+Time11);
	System.out.println("FLUKTUIEREND LinkedList: \t"+Time12);
	System.out.println("FLUKTUIEREND Vector:     \t"+Time13);
	System.out.println("FLUKTUIEREND+DELETE-VON-HINTEN ArrayList: \t"+Time21);
	System.out.println("FLUKTUIEREND+DELETE-VON-HINTEN LinkedList: \t"+Time22);
	System.out.println("FLUKTUIEREND+DELETE-VON-HINTEN Vector:     \t"+Time23);
	System.out.println("INSERT+LESEN ArrayList: \t"+Time31);
	System.out.println("INSERT+LESEN LinkedList: \t"+Time32);
	System.out.println("INSERT+LESEN Vector:     \t"+Time33);
    }	

    /**
     * Analysieren Sie, warum die Implementierungen sich unterscheiden. 
     * Wieso ist LinkedList schneller als ArrayList?
     * Achtung: Vector verwendet wohl eine andere Blockgröße als ArrayList
     */
    public static long grosseBestellung (String whichImplementation) {
	int i = 0;
	long startTime = System.nanoTime();

	Artikel tisch = new Artikel("Tisch",200);
	Artikel stuhl = new Artikel("Stuhl",100);
	Artikel schrank = new Artikel("Schrank",300);
	
	Bestellung1SpeedTest b1 = new Bestellung1SpeedTest("TUD",whichImplementation);
	b1.neuePosition(new Bestellposition(tisch,1));
	b1.neuePosition(new Bestellposition(schrank,2));
	
	// 5000 einzelne Positionen Stühle addieren
	// Performance Test der add()-Operation 
	while (i < 5000) {
	    b1.neuePosition(new Bestellposition(stuhl,1));
	    i++;
	}
// 	b1.print();
	
	long endTime = System.nanoTime();
	return endTime - startTime;
    }
    /**
     * Analysieren Sie, warum die Implementierungen sich unterscheiden. 
     * Wieso ist LinkedList wieder schneller? Warum ist das Löschen an Position 0
     * langsam mit ArrayList?
     */
    public static long fluktuierendeBestellung (String whichImplementation) {
	int i = 0;
	long startTime = System.nanoTime();

	Artikel tisch = new Artikel("Tisch",200);
	Artikel stuhl = new Artikel("Stuhl",100);
	Artikel schrank = new Artikel("Schrank",300);
	
	Bestellung1SpeedTest b1 = new Bestellung1SpeedTest("TUD",whichImplementation);
	b1.neuePosition(new Bestellposition(tisch,1));
	b1.neuePosition(new Bestellposition(schrank,2));
	
	// 5000 einzelne Positionen Stühle addieren
	// Performance Test der add()-Operation 
	while (i < 5000) {
	    b1.neuePosition(new Bestellposition(stuhl,1));
	    i++;
	}
	

	// 5000 einzelne Positionen Stühle löschen, immer von vorne löschen
	// Performance Test der remove()-Operation 
	i = 0;
	while (i < 5000) {
	    b1.loeschePosition(0);
	    i++;
	}
	// Zeige, dass Liste der Stühle leer ist
 	b1.print();
	
	long endTime = System.nanoTime();
	return endTime - startTime;
    }

    /**
     * Analysieren Sie, warum die Implementierungen sich unterscheiden. 
     * Wieso ist hier ArrayList schneller als LinkedList? Wieso geht das Löschen von
     * Hinten für ArrayList schneller?
     */
    public static long fluktuierendeBestellungMitLoeschenVonHinten (String whichImplementation) {
	int i = 0;
	long startTime = System.nanoTime();

	Artikel tisch = new Artikel("Tisch",200);
	Artikel stuhl = new Artikel("Stuhl",100);
	Artikel schrank = new Artikel("Schrank",300);
	
	Bestellung1SpeedTest b1 = new Bestellung1SpeedTest("TUD",whichImplementation);
	b1.neuePosition(new Bestellposition(tisch,1));
	b1.neuePosition(new Bestellposition(schrank,2));
	
	// 5000 einzelne Positionen Stühle addieren
	// Performance Test der add()-Operation 
	while (i < 5000) {
	    b1.neuePosition(new Bestellposition(stuhl,1));
	    i++;
	}
	
	// 5000 einzelne Positionen Stühle löschen, immer von hinten löschen
	// Performance Test der remove()-Operation 
	i = 0;
	while (i < 5000) {
	    // Achtung: das letzte Element wird adressiert mit size()-1 !!
	    b1.loeschePosition(b1.size()-1);
	    i++;
	}
	// Zeige, dass Liste der Stühle leer ist
 	b1.print();
	
	long endTime = System.nanoTime();
	return endTime - startTime;
    }
    /**
     * Wieso ist LinkedList hier langsamer als ArrayList, obwohl ArrayList langsamer aufbaut? 
     * (Hinweis: überlege, was die Verstreuung der Listenelemente über die Halde ausmacht)
     * Achtung: Hier sieht man, dass Vector eine andere Blockgröße als ArrayList verwendet,
     * denn es liest langsamer als ArrayList (vgl. Differenz)
     */
    public static long grosseBestellungMitLesen (String whichImplementation) {
	int i = 0;
	long startTime = System.nanoTime();

	Artikel tisch = new Artikel("Tisch",200);
	Artikel stuhl = new Artikel("Stuhl",100);
	Artikel schrank = new Artikel("Schrank",300);
	
	Bestellung1SpeedTest b1 = new Bestellung1SpeedTest("TUD",whichImplementation);
	b1.neuePosition(new Bestellposition(tisch,1));
	b1.neuePosition(new Bestellposition(schrank,2));
	
	// 5000 einzelne Positionen Stühle addieren
	// Performance Test der add()-Operation 
	while (i < 5000) {
	    b1.neuePosition(new Bestellposition(stuhl,1));
	    i++;
	}

	// 5000 einzelne Positionen lesen
	i = 0;
	while (i < 5000) {
	    Bestellposition b = b1.getPosition(i);
	    i++;
	}
//  	b1.print();
	
	long endTime = System.nanoTime();
	return endTime - startTime;
    }
}
